export interface INodeCallback<N, R> {
    /**
     * @param acc {Object[]} node's children transformation results
     * @param leaf {Object} current node
     * @returns transformation result
     */
    (acc: R[], node: N): R;
}
export interface ILeafCallback<L, R> {
    /**
     * @param leaf {Object} current leaf
     * @returns transformation result
     */
    (leaf: L): R;
}
export interface IChildrenSelector<N, L> {
    /**
     * @param node {Object} node
     * @returns {Object[]} array of children
     */
    (node: N): Array<N | L>;
}
/**
 * Tree utils
 */
export declare class Tree<N, L> {
    private root;
    private selector;
    constructor(root: N | L, childrenSelector: string | IChildrenSelector<N, L>);
    /**
     * Depth-first fold (reduce) of a multi-child tree
     * @param nodeCallback {Function} callback for nodes
     * @param leafCallback {Function} callback for leaves
     * @returns result of callback for root node
     */
    fold<R>(nodeCallback: INodeCallback<N, R>, leafCallback: ILeafCallback<L, R>): R;
    /**
     * Depth-first fold (reduce) of a multi-child tree
     */
    reduce<R>(nodeCallback: INodeCallback<N, R>, leafCallback: ILeafCallback<L, R>): R;
}
export declare module Tree {
    /**
     * Creates a multi-child tree from source
     * @param root {Object} root node
     * @param childrenSelector {string|Function} children array selector
     */
    function from<N, L>(root: N | L, childrenSelector: string | IChildrenSelector<N, L>): Tree<N, L>;
}
