"use strict";
/**
 * Tree utils
 */
var Tree = (function () {
    function Tree(root, childrenSelector) {
        this.root = root;
        this.selector = typeof childrenSelector === "string" ? function (x) { return x[childrenSelector]; } : childrenSelector;
        this.fold = this.reduce;
    }
    /**
     * Depth-first fold (reduce) of a multi-child tree
     */
    Tree.prototype.reduce = function (nodeCallback, leafCallback) {
        // 1. Prepare depth-first queue
        var _a = this, selector = _a.selector, root = _a.root;
        var preQ = [root];
        var postQ = [];
        // preparing depth-first query
        while (preQ.length) {
            var current = preQ.pop();
            var children = selector(current);
            if (children && children.length) {
                preQ.push.apply(preQ, children);
            }
            postQ.push(current);
        }
        // 2. Walking depth-first queue
        var acc = [];
        for (var i = postQ.length - 1; i >= 0; i--) {
            var current = postQ[i];
            var children = selector(current);
            if (children instanceof Array) {
                var clen = children.length;
                var subtree_acc = clen ? acc.splice(-clen, clen) : [];
                acc.push(nodeCallback(subtree_acc, current));
            }
            else {
                acc.push(leafCallback(current));
            }
        }
        // 3. Sanity check & return
        if (acc.length === 1) {
            return acc[0];
        }
        else {
            throw "Something went wrong: subtree accumulator has " + (acc.length - 1) + " leftover elements";
        }
    };
    return Tree;
}());
exports.Tree = Tree;
var Tree;
(function (Tree) {
    /**
     * Creates a multi-child tree from source
     * @param root {Object} root node
     * @param childrenSelector {string|Function} children array selector
     */
    function from(root, childrenSelector) {
        return new Tree(root, childrenSelector);
    }
    Tree.from = from;
})(Tree = exports.Tree || (exports.Tree = {}));
